// 
// (c)2020 by Lucky Resistor. See LICENSE for details.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

// The number of units in the x-axis
count_x = 2; // [1:5]
// The number of units in the y-axis
count_y = 1; // [1:3]
// The working height for the insert.
insert_working_height = 3; // [2:40]

// Module to hide system parameters from customizer.
module createInsert(count_x, count_y, insert_working_height) {
    
// Storage Box System Parameters
raster_x = 60;
raster_y = 60;
raster_z = 20;
wall_thickness = 1.67;
side_spacing = 0.5;
stacking_spacing = 0.1;
label_area = 8.8;

// Insert parameters
insert_fill_height = 4.8;
insert_spacing = 0.2;
insert_cr = 1; // The corner radius.
tec_groove_height = 5.8;

// Precalculated values
wall_inset = 2*wall_thickness + stacking_spacing + side_spacing/2;
insert_x = raster_x*count_x - 2*wall_inset;
insert_y = raster_y*count_y - 2*wall_inset;
insert_z = insert_fill_height + insert_working_height;

// Create a plain box with rounded corners at the end.
module createPlainBox() {
    translate([wall_inset, wall_inset, 0]) {
        union() {
            translate([insert_x-insert_cr, insert_cr, 0]) {
                cylinder(h=insert_z, r=insert_cr);
            }
            translate([insert_x-insert_cr, insert_y-insert_cr]) {
                cylinder(h=insert_z, r=insert_cr);
            }
            cube(size=[insert_x-insert_cr, insert_y, insert_z]);
            translate([insert_x-insert_cr-1, insert_cr, 0]) {    
                cube(size=[insert_cr+1, insert_y-2*insert_cr, insert_z]);
            }
        }
    }
}

// Create a grid with the grooves to remove
module createGrooves() {
    module createGroove(length) {
        rotate([90, 0, 180]) {
            linear_extrude(height=length) {
                polygon(points=[
                        [-tec_groove_height, 0],
                        [0, tec_groove_height],
                        [tec_groove_height, 0],
                        [tec_groove_height, -1],
                        [-tec_groove_height, -1]],
                    paths=[
                        [0, 1, 2, 3, 4]]);
            }
        }
    }
    for (i = [0:count_x]) {
        translate([raster_x*i, 0, 0]) {
            createGroove(raster_y*count_y);
        }
    }
    for (i = [0:count_y]) {
        translate([0, raster_y*i, 0]) {
            rotate([0, 0, -90]) {
                createGroove(raster_x*count_x);
            }
        }
    }
}

// Create the ribs to create the required removals there.
module createRibs() {
    module createRib(size) {
        translate([-wall_thickness/2-insert_spacing, 0, 0]) {
            cube(size=[wall_thickness+2*insert_spacing, wall_inset + size + insert_spacing, raster_z*4]);
        }
    }
    for (i = [0:count_x]) {
        translate([raster_x*i, 0, 0]) {
            createRib(2);
        }
        translate([raster_x*i, raster_y*count_y, 0]) {
            rotate([0, 0, 180]) {
                createRib(2);
            }
        }
    }
    for (i = [0:count_y]) {
        translate([raster_x*count_x, raster_y*i, 0]) {
            rotate([0, 0, 90]) {
                createRib(2);
            }
        }
    }
}

// Create the removal for the label area.
module createLabelArea() {
    length = raster_y * count_y;
    translate([0, length, 0]) {
        rotate([90, 0, 0]) {
            linear_extrude(height=length) {
                polygon(points=[
                        [0, 0],
                        [0, raster_z*4],
                        [label_area, raster_z*4],
                        [label_area, 1],
                        [label_area+1, 0],
                        [label_area+1, -1],
                        [0, -1]
                    ],
                    paths=[
                        [0, 1, 2, 3, 4, 5, 6]
                    ]);
            }
        }
    }
}

difference() {
    createPlainBox();
    createGrooves();
    createRibs();
    createLabelArea();
}


}


createInsert(count_x, count_y, insert_working_height);

